# -*- coding: utf-8 -*-
"""
Create, Read, Update, Delete functionality

"""

import sys
from sqlalchemy.orm import Session
from sqlalchemy.engine import Engine
from fastapi import File, UploadFile
from fastapi.responses import JSONResponse
from . import models, schemas
from toardb.variables import models as variables_models
from toardb.stationmeta import models as stationmeta_models
from toardb.stationmeta.crud import get_stationmeta_core
from toardb.timeseries.crud import get_timeseries_by_unique_constraints
import datetime as dt
import pandas as pd
from io import StringIO
from toardb.utils.database import engine

def get_data(db: Session, timeseries_id: int):
    return db.query(models.Data).filter(models.Data.timeseries_id == timeseries_id).all()


def get_data_by_datetime_and_timeseriesid(db: Session, datetime: dt.datetime, timeseries_id: int):
    return db.query(models.Data).filter([models.Data.datetime== datetime, models.Data.timeseries_id == timeseries_id]).first()


def get_all_data(db: Session, skip : int = 0, limit: int = None):
    return db.query(models.Data).offset(skip).limit(limit).all()


def create_data(db: Session, input_handle: UploadFile = File(...)):
    # a timeseries is defined by the unique_constraint of (station_id, variable_id, label)
    # station_id: from header
    # variable_id: from database (with variable_name -- from filename)
    # label: unique information for *this* timeseries (station_id, variable_id) -- at the moment: contributor_shortname 
    # get variable_name from filename
    variable_name = input_handle.filename.split('_')[0]
    variable = db.query(variables_models.Variable).filter(variables_models.Variable.name == variable_name).first()
    variable_id = variable.id
    # get header information (station_id, contributor_shortname)
    line = '#bla'
    f = input_handle.file
    prev = pos = 0
    while line[0] == '#':
        line = f.readline().decode('utf-8')
        key = line.split(':')[0].lower().strip()
        if key == "#station_id":
            station_id = line.split(':')[1]
        if key == "#contributor_shortname":
            label = line.split(':')[1]
        prev, pos = pos, f.tell()
    f.seek(prev)
    stationmeta_core = get_stationmeta_core(db=db,station_code=station_id)
    station_id = stationmeta_core.id
    timeseries = get_timeseries_by_unique_constraints(db=db,station_id=station_id,variable_id=variable_id,label=label)
    timeseries_id = timeseries.id
    # open SpooledTemporaryFile, skip header (and also try to insert timeseries_id!)
    df = pd.read_csv(input_handle.file, comment='#', header=None, sep=';',names=["time","value","flags"],parse_dates=["time"],index_col="time")
    # now insert the timeseries_id to the end of the data frame
    df.insert(2, 'timeseries_id', timeseries_id)
    # datetime needs timezone information
    df = df.tz_localize('UTC')
    buf = StringIO()
    df.to_csv(buf, header=False)
    buf.pos = 0
    buf.seek(0)
    fake_conn = engine.raw_connection()
    fake_cur = fake_conn.cursor()
    # I really want to return whether the command worked or not! (this issue is now still open)
    try:
        fake_cur.copy_from(buf, 'data', sep=',', columns=('datetime','value','flags','timeseries_id'))
        fake_conn.commit()
        message = "Data successfully inserted."
    except:
        e = sys.exc_info()[0]
        message = "An error occurred in data insertion: %s" % (e,)
    return JSONResponse(message)

